// array.cpp
// Array class template example using static allocation
// See Exercise 12.7

#include <iostream.h>
#include <stdlib.h>
#include <assert.h>

template < class elementType, int numberOfElements >
class Array {
public:
   friend ostream &operator<<( ostream &, const Array & );
   Array();                                // default constructor
   ~Array();                               // destructor
   int getSize() const;                    // return size
   bool operator==( const Array & ) const; // compare equal
   static int getArrayCount();             // Return count of arrays
   void inputArray();                      // input the array elements
private:
   elementType ptr[ numberOfElements ]; // pointer to first element of array
   int size; // size of the array
   static int arrayCount;  // # of Arrays instantiated
};
// Initialize static data member at file scope
template < class elementType, int numberOfElements >
int Array< elementType, numberOfElements >::arrayCount = 0;   // no objects yet

// Default constructor for class Array
template < class elementType, int numberOfElements >
Array< elementType, numberOfElements >::Array()
{
   ++arrayCount;                 // count one more object
   size = numberOfElements;
   cout << "Initializing array #" << arrayCount << ": ";
   cout << size << " elements; ";
   cout << "element size = " << sizeof( elementType );
   cout << " bytes.\n";
   for ( int i = 0; i < size; ++i )
      ptr[ i ] = 0;                // initialize array
}

// Destructor for class Array
template < class elementType, int numberOfElements >
Array< elementType, numberOfElements >::~Array() { --arrayCount; }

// Get the size of the array
template < class elementType, int numberOfElements >
int Array< elementType, numberOfElements >::getSize() const { return size; }

// Return the number of Array objects instantiated
template < class elementType, int numberOfElements >
int Array< elementType, numberOfElements >::getArrayCount() 
   { return arrayCount; }

// Determine if two arrays are equal and
// return true or false.
template < class elementType, int numberOfElements >
bool Array< elementType, numberOfElements >::
            operator==( const Array &right ) const
{
   if ( size != right.size )
      return false;    // arrays of different sizes

   for ( int i = 0; i < size; ++i )
      if ( ptr[ i ] != right.ptr[ i ] )
         return false; // arrays are not equal

   return true;       // arrays are equal
}

// Input values for entire array.
template < class elementType, int numberOfElements >
void Array< elementType, numberOfElements >::inputArray()
{
   for ( int i = 0; i < size; ++i )
      cin >> ptr[ i ];
}

// Overload output operator as a non-member friend
template < class T, int N >
ostream &operator<<( ostream & output, const Array< T, N > &a )
{  int i;
   for (i=0; i<a.size; i++)
      output << a.ptr[ i ] << " ";
   return output << endl;
}

// driver program to test static Array template classes
int main()
{
   // Test constructors and counter functions
   Array< short, 5 > short1;	// declare short short1[5]
   cout << short1.getArrayCount() << " array(s) declared\n";
   cout << "short1[] = " << short1 << endl;

   // Input values for short1[]
   cout << "Enter " << short1.getSize() << " values for short1[]:\n";
   short1.inputArray();
   cout << "short1[] = " << short1 << endl;

   Array< short, 5 > short2;	// declare short short2[5]

   // Input values for short2[]
   cout << "Enter " << short2.getSize() << " values for short2[]:\n";
   short2.inputArray();
   cout << "short2[] = " << short2 << endl;

   // Test == operator
   if ( short1 == short2 )
      cout << "short1[] == short2[]\n";
   else
      cout << "short1[] != short2[]\n";

   return 0;
}

// output begins here

Initializing array #1: 5 elements; element size = 2 bytes.
1 array(s) declared
short1[] = 0 0 0 0 0

Enter 5 values for short1[]:
1 2 3 4 5
short1[] = 1 2 3 4 5

Initializing array #2: 5 elements; element size = 2 bytes.
Enter 5 values for short2[]:
1 2 3 4 5
short2[] = 1 2 3 4 5

short1[] == short2[]

